--------------------------------------------------------
--  DDL for View V_CANDIDATURES_ACCEPTADES
--------------------------------------------------------

  CREATE OR REPLACE FORCE VIEW "MAGATZEM"."V_CANDIDATURES_ACCEPTADES" ("ID_OFERTA", "ID_EMPRESA_OFERTA", "ID_CANDIDAT_CV", "ID_CV_CANDIDATURA", "DATA_ALTA", "DATA_ACCEPTACIO") AS 
  SELECT candidatura.id_oferta_candidatura id_oferta,
    candidatura.id_empresa_oferta id_empresa_oferta,
    candidatura.id_candidat_cv,
    candidatura.id_cv_candidatura,
    candidatura.data_alta data_alta,
    candidatura.data_acceptacio data_acceptacio
  FROM raw_mag.candidatura candidatura
  WHERE candidatura.estat_candidatura=3;
--------------------------------------------------------
--  DDL for View V_ESTAT_CAND_OFERTA
--------------------------------------------------------

  CREATE OR REPLACE FORCE VIEW "MAGATZEM"."V_ESTAT_CAND_OFERTA" ("ID_OFERTA_CANDIDATURA", "ID_EMPRESA_OFERTA", "REBUTJADES", "ENVIADES", "ENPROCES", "ACCEPTADES") AS 
  Select id_oferta_candidatura,
       id_empresa_oferta,
       SUM(Rebutjades) Rebutjades,
       SUM(Enviades) Enviades,
       SUM(EnProces) EnProces,
       SUM(Acceptades) Acceptades
FROM (       
   select id_oferta_candidatura,
       id_empresa_oferta,
       estat_candidatura,
       decode(estat_candidatura,0,1,0) Rebutjades,
       decode(estat_candidatura,1,1,0) Enviades,
       decode(estat_candidatura,2,1,0) EnProces,
       decode(estat_candidatura,3,1,0) Acceptades
   from raw_mag.candidatura
)
group by id_oferta_candidatura,id_empresa_oferta;
--------------------------------------------------------
--  DDL for View V_EXPERIENCIES_ACCEPTADES
--------------------------------------------------------

  CREATE OR REPLACE FORCE VIEW "MAGATZEM"."V_EXPERIENCIES_ACCEPTADES" ("ID_OFERTA", "ID_EMPRESA_OFERTA", "ID_CANDIDAT_CV", "ID_CV_CANDIDATURA", "EXPERIENCIES") AS 
  select oferta.id_oferta,
       oferta.id_empresa_oferta,
       candidatura.id_candidat_cv, 
       candidatura.id_cv_candidatura,
       count(exp.id_experiencia) experiencies
from raw_mag.oferta oferta,
     raw_mag.candidatura candidatura,
     raw_mag.cv_experiencia exp
where oferta.id_oferta=candidatura.id_oferta_candidatura and
      oferta.id_empresa_oferta =candidatura.id_empresa_oferta and
      candidatura.estat_candidatura=3 and
      candidatura.id_candidat_cv=exp.id_candidat_cv and
      candidatura.id_cv_candidatura=exp.id_cv
group by  oferta.id_oferta,
       oferta.id_empresa_oferta,
       candidatura.id_candidat_cv, 
       candidatura.id_cv_candidatura;
--------------------------------------------------------
--  DDL for Function F_CALCULAR_FETS_OFERTA
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CALCULAR_FETS_OFERTA" RETURN NUMBER AS
/*
Aquesta funci calcula les dades de la taula de fets de Oferta
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Oferta is
select ID_EMPRESA_OFERTA,
       ID_OFERTA,
       DATA_ALTA,
       NOMBRE_VACANTS_OFERTA,
       SALARI_MINIM_OFERTA,
       SALARI_MAXIM_OFERTA,
       TIPUS_CONTRACTE_OFERTA,
       TIPUS_JORNADA_OFERTA,
       CIUTAT_TREBALL_OFERTA,
       ROL_CANDIDAT_OFERTA,
       DATA_LIMIT,
       DESCRIPCIO_CURTA_OFERTA,
       ID_PROVINCIA_CIUTAT
from raw_mag.oferta;

/*Variables contenidores de paramtres*/
v_ID_EMPRESA_OFERTA oferta.ID_EMPRESA_OFERTA%TYPE;
v_ID_OFERTA oferta.ID_OFERTA%TYPE;
v_DATA_ALTA oferta.DATA_ALTA_OFERTA%TYPE;
v_NOMBRE_VACANTS_OFERTA oferta.NOMBRE_VACANTS_OFERTA%TYPE;
v_SALARI_MINIM_OFERTA oferta.SALARI_MINIM_OFERTA%TYPE;
v_SALARI_MAXIM_OFERTA oferta.SALARI_MAXIM_OFERTA%TYPE;
v_TIPUS_CONTRACTE_OFERTA oferta.TIPUS_CONTRACTE_OFERTA%TYPE;
v_TIPUS_JORNADA_OFERTA oferta.TIPUS_JORNADA_OFERTA%TYPE;
v_CIUTAT_TREBALL_OFERTA oferta.CIUTAT_TREBALL_OFERTA%TYPE;
v_ROL_CANDIDAT_OFERTA oferta.ROL_CANDIDAT_OFERTA%TYPE;
v_DATA_LIMIT oferta.DATA_LIMIT%TYPE;
v_DESCRIPCIO_CURTA_OFERTA oferta.DESCRIPCIO_CURTA_OFERTA%TYPE;
v_ID_PROVINCIA_CIUTAT oferta.ID_PROVINCIA_CIUTAT%TYPE;
v_CANDIDAT_TROBAT OFERTA.CANDIDAT_TROBAT%TYPE;
v_DIES_SELECCIO oferta.DIES_SELECCIO%TYPE;
v_EXPERIENCIES_ACCEPTAT oferta.EXPERIENCIES_ACCEPTAT%TYPE;
v_CANDIDATURES_ENVIADES oferta.CANDIDATURES_ENVIADES%TYPE;
v_CANDIDATURES_REBUTJADES oferta.CANDIDATURES_REBUTJADES%TYPE;
v_CANDIDATURES_ACCEPTADES oferta.CANDIDATURES_ACCEPTADES%TYPE;
v_CANDIDATURES_EN_PROCES oferta.CANDIDATURES_EN_PROCES%TYPE;
v_ID_SECTOR oferta.ID_SECTOR%TYPE;
v_SUB_SECTOR_EMPRESA oferta.SUB_SECTOR_EMPRESA%TYPE;
v_CANDIDATS_TOTALS oferta.CANDIDATS_TOTALS%TYPE;

v_aux_comptador NUMBER;
v_Primera_Acceptacio DATE;

/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='OFERTA';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Oferta;
FETCH c_Oferta into v_ID_EMPRESA_OFERTA,
                    v_ID_OFERTA,
                    v_DATA_ALTA, 
                    v_NOMBRE_VACANTS_OFERTA,
                    v_SALARI_MINIM_OFERTA,
                    v_SALARI_MAXIM_OFERTA,
                    v_TIPUS_CONTRACTE_OFERTA,
                    v_TIPUS_JORNADA_OFERTA,
                    v_CIUTAT_TREBALL_OFERTA,
                    v_ROL_CANDIDAT_OFERTA,
                    v_DATA_LIMIT,
                    v_DESCRIPCIO_CURTA_OFERTA,
                    v_ID_PROVINCIA_CIUTAT;



WHILE c_Oferta%FOUND LOOP
    -- Anem a calcular candidat_trobat
    select count(*) into v_CANDIDAT_TROBAT
    from V_EXPERIENCIES_ACCEPTADES
    WHERE ID_EMPRESA_OFERTA= v_ID_EMPRESA_OFERTA and ID_OFERTA=v_ID_OFERTA;
    IF(v_CANDIDAT_TROBAT>0) THEN v_CANDIDAT_TROBAT:=1;END IF;
        
    v_DIES_SELECCIO:=0;
    v_EXPERIENCIES_ACCEPTAT:=0;
    
    select count(*) into v_aux_comptador
    from v_candidatures_acceptades
    where id_oferta=v_ID_OFERTA and id_empresa_oferta=v_ID_EMPRESA_OFERTA;
    
    IF (v_aux_comptador>0) THEN
    --Calculem dies seleccio. El nombre de dies de l'alta de la oferta a la primera acceptacio
      select min(DATA_acceptacio) into v_Primera_Acceptacio
      from   V_CANDIDATURES_ACCEPTADES
      WHERE ID_EMPRESA_OFERTA= v_ID_EMPRESA_OFERTA and ID_OFERTA=v_ID_OFERTA
      group by ID_OFERTA, ID_EMPRESA_OFERTA;
      IF (v_Primera_Acceptacio is null) THEN v_DIES_SELECCIO:=0; 
      ELSE 
          v_DIES_SELECCIO:= to_date(v_Primera_Acceptacio,'DD/MM/YYYY') - to_date(v_DATA_ALTA,'DD/MM/YYYY');
      END IF;    
      
    -- Calculem la suma de les experients dels candidats acceptats
       IF (v_CANDIDAT_TROBAT>0) THEN
          select sum(experiencies) into v_EXPERIENCIES_ACCEPTAT 
          from v_experiencies_acceptades
          where ID_EMPRESA_OFERTA= v_ID_EMPRESA_OFERTA and ID_OFERTA=v_ID_OFERTA;
       ELSE
          v_EXPERIENCIES_ACCEPTAT:=0;
       END IF;   
    END IF;
    
    -- Calculem 
    -- CANDIDATURES_ENVIADES,CANDIDATURES_REBUTJADES,CANDIDATURES_ACCEPTADES i 
    -- CANDIDATURES_EN_PROCES*/
    --Primer mirem que hi hagi candidatures associades
    v_CANDIDATURES_ENVIADES:=0;
    v_CANDIDATURES_REBUTJADES:=0;
    v_CANDIDATURES_ACCEPTADES:=0;
    v_CANDIDATURES_EN_PROCES:=0;
    v_CANDIDATS_TOTALS:=0;
    select count(*) into v_aux_comptador
    from raw_mag.candidatura
    where id_empresa_oferta=v_ID_EMPRESA_OFERTA and 
    id_oferta_candidatura=v_ID_OFERTA;
    --Com l'oferta te candidatures anem a mirar en quin estat estan
    IF (v_aux_comptador>0) THEN
      Select Rebutjades,Enviades,EnProces,Acceptades into
             v_CANDIDATURES_REBUTJADES,
             v_CANDIDATURES_ENVIADES,
             v_CANDIDATURES_EN_PROCES,
             v_CANDIDATURES_ACCEPTADES
      from v_estat_cand_oferta
      where id_empresa_oferta=v_ID_EMPRESA_OFERTA and 
            id_oferta_candidatura=v_ID_OFERTA;
    v_CANDIDATS_TOTALS:= v_CANDIDATURES_REBUTJADES+v_CANDIDATURES_ENVIADES+
             v_CANDIDATURES_EN_PROCES+ v_CANDIDATURES_ACCEPTADES;       
    END IF;
    
    --Recuperem el sector i sub-sector de l'empresa
    select sector_empresa,sub_sector_empresa into v_ID_SECTOR,v_SUB_SECTOR_EMPRESA
    from RAW_MAG.empresa
    where id_empresa=v_ID_EMPRESA_OFERTA;
    
    --Construccio de la sentncia de l'insert de la oferta
    v_sentencia:='insert into oferta (ID_EMPRESA_OFERTA,ID_OFERTA,'||
                'DATA_ALTA_OFERTA,NOMBRE_VACANTS_OFERTA,SALARI_MINIM_OFERTA,'||
                'SALARI_MAXIM_OFERTA,TIPUS_CONTRACTE_OFERTA,'||
                'TIPUS_JORNADA_OFERTA,CIUTAT_TREBALL_OFERTA,'||
                'ROL_CANDIDAT_OFERTA,DATA_LIMIT,DESCRIPCIO_CURTA_OFERTA,'||
                'ID_PROVINCIA_CIUTAT,CANDIDAT_TROBAT,COMPTADOR,DIES_SELECCIO,'||
                'EXPERIENCIES_ACCEPTAT,CANDIDATURES_ENVIADES,'||
                'CANDIDATURES_REBUTJADES,CANDIDATURES_ACCEPTADES,'||
                'CANDIDATURES_EN_PROCES,ID_SECTOR,SUB_SECTOR_EMPRESA,'||
                'CANDIDATS_TOTALS) values ('||
                  v_ID_EMPRESA_OFERTA||','||
                  v_ID_OFERTA||','||
                  'to_date('''||v_DATA_ALTA||''',''DD/MM/YYYY''),'||
                  v_NOMBRE_VACANTS_OFERTA||','||
                  v_SALARI_MINIM_OFERTA||','||
                  v_SALARI_MAXIM_OFERTA||','||
                  v_TIPUS_CONTRACTE_OFERTA||','||
                  v_TIPUS_JORNADA_OFERTA||','||
                  v_CIUTAT_TREBALL_OFERTA||','||
                  v_ROL_CANDIDAT_OFERTA||','||
                  'to_date('''||v_DATA_LIMIT||''',''DD/MM/YYYY''),'||
                  ''''||v_DESCRIPCIO_CURTA_OFERTA||''','||
                  v_ID_PROVINCIA_CIUTAT||','||
                  v_CANDIDAT_TROBAT||','||
                  '1,'||
                  v_DIES_SELECCIO||','||
                  v_EXPERIENCIES_ACCEPTAT||','||
                  v_CANDIDATURES_ENVIADES||','||
                  v_CANDIDATURES_REBUTJADES||','||
                  v_CANDIDATURES_ACCEPTADES||','||
                  v_CANDIDATURES_EN_PROCES||','||
                  v_ID_SECTOR||','||
                  v_SUB_SECTOR_EMPRESA||','||
                  v_CANDIDATS_TOTALS||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
    FETCH c_Oferta into v_ID_EMPRESA_OFERTA,
                    v_ID_OFERTA,
                    v_DATA_ALTA, 
                    v_NOMBRE_VACANTS_OFERTA,
                    v_SALARI_MINIM_OFERTA,
                    v_SALARI_MAXIM_OFERTA,
                    v_TIPUS_CONTRACTE_OFERTA,
                    v_TIPUS_JORNADA_OFERTA,
                    v_CIUTAT_TREBALL_OFERTA,
                    v_ROL_CANDIDAT_OFERTA,
                    v_DATA_LIMIT,
                    v_DESCRIPCIO_CURTA_OFERTA,
                    v_ID_PROVINCIA_CIUTAT;
END LOOP;
CLOSE c_Oferta;    
dbms_output.put_line('F_CALCULAR_FETS_OFERTA: Taula Oferta carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Oferta: Nombre d''errors al carregar fets massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar oferta: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CALCULAR_FETS_OFERTA;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_CIUTAT
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_CIUTAT" RETURN NUMBER AS 
/*
Aquesta funci carrega la ciutat de la funci geografica
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Ciutat is
  select id_provincia_ciutat,id_ciutat,id_pais_provincia,desc_ciutat
  from RAW_MAG.ciutat;

/*Variables contenidores de paramtres*/
v_id_provincia_ciutat ciutat.id_provincia_ciutat%TYPE;
v_id_ciutat ciutat.id_ciutat%TYPE;
v_id_pais_provincia ciutat.id_pais_provincia%TYPE;
v_desc_ciutat ciutat.desc_ciutat%TYPE;

/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='PAIS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Ciutat;
FETCH c_Ciutat into v_id_provincia_ciutat,v_id_ciutat,v_id_pais_provincia,
                    v_desc_ciutat;
WHILE c_Ciutat%FOUND LOOP
    v_sentencia:='insert into ciutat (id_provincia_ciutat,id_ciutat,id_pais_provincia,desc_ciutat) values ('||
                  v_id_provincia_ciutat||','||
                  v_id_ciutat||','||
                  v_id_pais_provincia||','||
                  ''''||v_desc_ciutat||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
    FETCH c_Ciutat into v_id_provincia_ciutat,v_id_ciutat,v_id_pais_provincia,
                        v_desc_ciutat;
END LOOP;
CLOSE c_Ciutat;  
dbms_output.put_line('F_CARREGAR_CIUTAT: Taula Ciutat carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Ciutat: Nombre d''errors al carregar la dimensio Provincia massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Ciutat: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CARREGAR_CIUTAT;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_DIMENSIONS
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_DIMENSIONS" RETURN NUMBER AS 
/*
Aquesta funci s'encarrega de carregar les dimensions del magatzem
*/
e_Massa_Errors EXCEPTION;
v_Resultat_Carrega NUMBER;
BEGIN

--Carreguem la dimensio temps
v_Resultat_Carrega:=f_carregar_temps;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Carreguem la dimensi empresa
v_Resultat_Carrega:=f_carregar_empresa;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Carreguem la dimensi Rols
v_Resultat_Carrega:=f_carregar_rol;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Carreguem la dimensi Geografica
--Comencem per Pais
v_Resultat_Carrega:=f_carregar_pais;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Ara les provincies
v_Resultat_Carrega:=f_carregar_provincia;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Finalment les ciutats
v_Resultat_Carrega:=f_carregar_ciutat;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Carreguem la dimensio sector
--Primer el nivell sector
v_Resultat_Carrega:=f_carregar_sector;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
--Ara el nivell sub sector
v_Resultat_Carrega:=f_carregar_sub_sector;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Carreguem la dimensio Tipus de Contracte
v_Resultat_Carrega:=f_carregar_tipus_contracte;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
--Carreguem la dimensio Tipus de Jornada
v_Resultat_Carrega:=f_carregar_tipus_jornada;
IF (v_Resultat_Carrega<>1) THEN RAISE e_Massa_Errors;END IF;
commit;
dbms_output.put_line('F_CARREGAR_DIMENSIONS:Dimensions carregades.');
return 1;
EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Dimensions: Nombre d''errors al carregar la dimensions massa gran. Abortar carrega magatzem');
        dbms_output.put_line('Anular carrega de magatzem. Problema dimensions');
        return -1;
                
  WHEN OTHERS THEN
        insertar_log('ABORTAR CARREGA TOTAL: carregar Empresa: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
        dbms_output.put_line('Anular carrega de magatzem. Problema dimensions');
        return -1;
END F_CARREGAR_DIMENSIONS;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_EMPRESA
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_EMPRESA" RETURN NUMBER AS
/*
Aquest funcio carrega la dimensio de empresa. Si el nombre d'errors de carrega
supera el maxim parametritzat s'abortar la crrega
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Empresa is
    select ID_EMPRESA,NOM_EMPRESA,CIF_EMPRESA,to_date(DATA_ALTA,'DD/MM/YYYY'),
           to_date(DATA_BAIXA,'DD/MM/YYYY'),NOMBRE_TREBALLADORS,
           ADRECA_WEB_EMPRESA
    from RAW_MAG.empresa;

/*Variables contenidores de paramtres*/
v_ID_EMPRESA EMPRESA.ID_EMPRESA%TYPE;
v_NOM_EMPRESA EMPRESA.NOM_EMPRESA%TYPE;
v_CIF_EMPRESA EMPRESA.CIF_EMPRESA%TYPE;
v_DATA_ALTA EMPRESA.DATA_ALTA%TYPE;
v_DATA_BAIXA EMPRESA.DATA_BAIXA%TYPE;
v_NOMBRE_TREBALLADORS EMPRESA.NOMBRE_TREBALLADORS%TYPE;
v_ADRECA_WEB_EMPRESA EMPRESA.ADRECA_WEB_EMPRESA%TYPE;
v_sentencia varchar2(2000);
BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='EMPRESA';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Empresa;
FETCH c_Empresa into v_ID_EMPRESA,v_NOM_EMPRESA,v_CIF_EMPRESA,v_DATA_ALTA,
                         v_DATA_BAIXA,v_NOMBRE_TREBALLADORS,
                         v_ADRECA_WEB_EMPRESA;
WHILE c_Empresa%FOUND LOOP                         
    v_sentencia:='insert into empresa (ID_EMPRESA,NOM_EMPRESA,CIF_EMPRESA,'||
                  'DATA_ALTA,DATA_BAIXA,NOMBRE_TREBALLADORS,ADRECA_WEB_EMPRESA)'|| 
                  'values ('||
                  v_ID_EMPRESA ||','||''''||v_NOM_EMPRESA||''''||
                  ','||''''||v_CIF_EMPRESA||''''||
                  ',to_date('''||v_DATA_ALTA||''',''DD/MM/YYYY''),'||
                  'to_date('''||v_DATA_BAIXA||''',''DD/MM/YYYY''),'||
                  v_NOMBRE_TREBALLADORS||','''||v_ADRECA_WEB_EMPRESA||''''||')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;  
    FETCH c_Empresa into v_ID_EMPRESA,v_NOM_EMPRESA,v_CIF_EMPRESA,v_DATA_ALTA,
                         v_DATA_BAIXA,v_NOMBRE_TREBALLADORS,
                         v_ADRECA_WEB_EMPRESA;
END LOOP;
CLOSE c_Empresa;  
dbms_output.put_line('F_CARREGAR_EMPRESA:Taula empresa carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Empresa: Nombre d''errors al carregar la dimensio empresa massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Empresa: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 

END F_CARREGAR_EMPRESA;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_PAIS
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_PAIS" RETURN NUMBER AS
/*Aquesta funcio carrega pais a la dimensi geografica*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Pais is
  select id_pais, desc_pais
  from RAW_MAG.pais;

/*Variables contenidores de paramtres*/
v_ID_PAIS  pais.id_pais%TYPE;
v_DESC_PAIS pais.desc_pais%TYPE;
/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='PAIS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Pais;
FETCH c_Pais into v_ID_PAIS,v_DESC_PAIS;
WHILE c_Pais%FOUND LOOP
    v_sentencia:='insert into pais (ID_PAIS,DESC_PAIS) values ('||
                  v_ID_PAIS||','||''''||v_DESC_PAIS||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
    FETCH c_Pais into v_ID_PAIS,v_DESC_PAIS;
END LOOP;
CLOSE c_Pais; 
dbms_output.put_line('F_CARREGAR_PAIS:Taula Pais carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Pais: Nombre d''errors al carregar la dimensio rols massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Pais: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CARREGAR_PAIS;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_PROVINCIA
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_PROVINCIA" RETURN NUMBER AS 
/*
Aquesta funci carrega la provincia de la dimensi geografica
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Provincia is
  select id_pais_provincia, id_provincia,desc_provincia
  from RAW_MAG.provincia;

/*Variables contenidores de paramtres*/
v_id_pais_provincia  provincia.id_pais_provincia%TYPE;
v_id_provincia  provincia.id_provincia%TYPE;
v_desc_provincia  provincia.desc_provincia%TYPE;
/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='PAIS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Provincia;
FETCH c_Provincia into v_id_pais_provincia,v_id_provincia,v_desc_provincia;
WHILE c_Provincia%FOUND LOOP
    v_sentencia:='insert into provincia (id_pais_provincia,id_provincia,desc_provincia) values ('||
                  v_id_pais_provincia||','||
                  v_id_provincia||','||
                  ''''||v_desc_provincia||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
    FETCH c_Provincia into v_id_pais_provincia,v_id_provincia,v_desc_provincia;
END LOOP;
CLOSE c_Provincia;  
dbms_output.put_line('F_CARREGAR_PROVINCIA:Taula Provincia carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Provincia: Nombre d''errors al carregar la dimensio Provincia massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Pais: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CARREGAR_PROVINCIA;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_ROL
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_ROL" RETURN NUMBER AS
/*
Aquesta funcio carrega la dimensio de rols
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Rols is
    select id_rol,desc_rol
    from RAW_MAG.mestre_rols;

/*Variables contenidores de paramtres*/
v_ID_ROL  mestre_rols.id_rol%TYPE;
v_DESC_ROL mestre_rols.desc_rol%TYPE;
/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='MESTRE_ROLS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Rols;
FETCH c_Rols into v_ID_ROL,v_DESC_ROL;
WHILE c_Rols%FOUND LOOP
    v_sentencia:='insert into mestre_rols (ID_ROL,DESC_ROL) values ('||
                  v_ID_ROL||','||''''||v_DESC_ROL||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
    FETCH c_Rols into v_ID_ROL,v_DESC_ROL;
END LOOP;
CLOSE c_Rols;   
dbms_output.put_line('F_CARREGAR_ROL:Taula Rol carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Rols: Nombre d''errors al carregar la dimensio rols massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar rols: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 


END F_CARREGAR_ROL;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_SECTOR
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_SECTOR" RETURN NUMBER AS 
/*
Aquesta funcio serveix per carregar el nivell de sector de la dimensio sector
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Sector is
   select id_sector, desc_sector
   from RAW_MAG.sector;

/*Variables contenidores de paramtres*/
v_id_sector  sector.id_sector%TYPE;
v_desc_sector sector.desc_sector%TYPE;
/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='PAIS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Sector;
FETCH c_Sector into v_id_sector, v_desc_sector;
WHILE c_Sector%FOUND LOOP
    v_sentencia:='insert into sector (id_sector, desc_sector) values ('||
                  v_id_sector||','||''''||v_desc_sector||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
   FETCH c_Sector into v_id_sector, v_desc_sector;
END LOOP;
CLOSE c_Sector;
dbms_output.put_line('F_CARREGAR_SECTOR:Taula Sector carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Sector: Nombre d''errors al carregar la dimensio Sector massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Sector: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CARREGAR_SECTOR;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_SUB_SECTOR
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_SUB_SECTOR" RETURN NUMBER AS
/*
Aquesta funcio serveix per carregar el nivell de SUB sector de la dimensio sector
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_Sub_Sector is
   select id_sector, id_sub_sector,desc_sub_sector
   from RAW_MAG.sub_sector;

/*Variables contenidores de paramtres*/
v_id_sector       sub_sector.id_sector%TYPE;
v_id_sub_sector   sub_sector.id_sub_sector%TYPE;
v_desc_sub_sector sub_sector.desc_sub_sector%TYPE;
/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='PAIS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_Sub_Sector;
FETCH c_Sub_Sector into v_id_sector, v_id_sub_sector,v_desc_sub_sector;
WHILE c_Sub_Sector%FOUND LOOP
    v_sentencia:='insert into sub_sector (id_sector, id_sub_sector,desc_sub_sector) values ('||
                  v_id_sector||','||
                  v_id_sub_sector||','||
                  ''''||v_desc_sub_sector||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
   FETCH c_Sub_Sector into v_id_sector, v_id_sub_sector,v_desc_sub_sector;
END LOOP;
CLOSE c_Sub_Sector; 
dbms_output.put_line('F_CARREGAR_SUB_SECTOR:Taula Sub_Sector carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Sub Sector: Nombre d''errors al carregar la dimensio Sector massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Sub Sector: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CARREGAR_SUB_SECTOR;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_TEMPS
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_TEMPS" RETURN NUMBER AS 
/* 
Aquesta funcio calcula la dimensi temps per complet
*/
v_data_ini DATE;
v_data_fi DATE;
v_data_aux DATE;

BEGIN
--Capturem la data inicial i la data final de la taula de parametres
select to_date(data_inicial,'DD/MM/YYYY'), to_date(data_final,'DD/MM/YYYY') into
       v_data_ini,v_data_fi
from control_dates;
v_data_aux:=v_data_ini;


--Desactivem les restriccions per no tenir problemes amb les claus foranies
DESFER_CONSTRAINTS;

--Primer la taula de dia
     WHILE v_data_aux<=v_data_fi LOOP
     insert into dia (DIA,NUMANY,NUM_MES,NUM_SETMANA,DESC_DIA,DIA_SETMANA) values(
            to_date(v_data_aux,'DD/MM/YYYY'),
            to_number(to_char(to_date(v_data_aux,'DD/MM/RRRR'),'YYYY')),
            to_number(to_char(v_data_aux,'MM')),
            to_number(to_char(v_data_aux,'WW')),
            to_char(to_date(v_data_aux)),
            to_number(to_char(v_data_aux,'D')));
            
            v_data_aux:=v_data_aux+1;
     END LOOP;
     
--Ara la taula de mes     
execute immediate('insert into mes select distinct numany,num_mes,to_char(dia,''MONTH'') from dia');

--Ara la taula de any
execute immediate('insert into numany select distinct numany,to_char(numany) from mes');

--Ara la taula de setmana
execute immediate('insert into setmana select distinct numany,to_number(to_char(dia,''WW'')),''Setmana numero ''|| to_char(dia,''WW'') from dia');

--Tornem a activar constants
FER_CONSTRAINTS;
dbms_output.put_line('F_CARREGAR_TEMPS:Taula Temps carregada.');
return 1;
EXCEPTION  
  WHEN OTHERS THEN
        insertar_log('ABORTAR CARREGA TOTAL: carregar temps: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
        dbms_output.put_line('Anular carrega de magatzem. Problema temps');
        return -1;

END F_CARREGAR_TEMPS;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_TIPUS_CONTRACTE
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_TIPUS_CONTRACTE" RETURN NUMBER AS 
/*
Aquesta funcio serveix per carregar la dimensio tipus de contracte
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_TContracte is
   select m_contracte, descripcio_contracte
   from RAW_MAG.m_tipus_contracte;

/*Variables contenidores de paramtres*/
v_m_contracte  tipus_contracte.m_contracte%TYPE;
v_descripcio_contracte tipus_contracte.descripcio_contracte%TYPE;
/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='PAIS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_TContracte;
FETCH c_TContracte into v_m_contracte, v_descripcio_contracte;
WHILE c_TContracte%FOUND LOOP
    v_sentencia:='insert into tipus_contracte (m_contracte, descripcio_contracte) values ('||
                  v_m_contracte||','||''''||v_descripcio_contracte||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
   FETCH c_TContracte into v_m_contracte, v_descripcio_contracte;
END LOOP;
CLOSE c_TContracte;    
dbms_output.put_line('F_CARREGAR_TIPUS_CONTRACTE:Taula Tipus_Contracte carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Tipus Contracte: Nombre d''errors al carregar la dimensio Tipus Contracte massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Tipus Contracte: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CARREGAR_TIPUS_CONTRACTE;

/

--------------------------------------------------------
--  DDL for Function F_CARREGAR_TIPUS_JORNADA
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_CARREGAR_TIPUS_JORNADA" RETURN VARCHAR2 AS 
/*
Aquesta funcio serveix per carregar la dimensio tipus de jornada
*/
e_Massa_Errors EXCEPTION;
v_Maxim_Errors NUMBER;
v_Errors_actual NUMBER;
v_carrega_actual NUMBER;
CURSOR c_TJornada is
   select m_jornada,descripcio_jornada
   from RAW_MAG.m_tipus_jornada;

/*Variables contenidores de paramtres*/
v_m_jornada  tipus_jornada.m_jornada%TYPE;
v_descripcio_jornada tipus_jornada.descripcio_jornada%TYPE;
/*Variable amb la sentencia*/
v_sentencia varchar2(2000);

BEGIN
/*Primer busquem el nombre mxim d'errors tolerables per la taula*/
select nombre_errors into v_Maxim_Errors
from control_errors
where nom_taula='PAIS';

/*Incialitzo els errors actuals*/
v_Errors_actual:=0;

/*Anem a carregar la taula*/
OPEN c_TJornada;
FETCH c_TJornada into v_m_jornada,v_descripcio_jornada;
WHILE c_TJornada%FOUND LOOP
    v_sentencia:='insert into tipus_jornada (m_jornada,descripcio_jornada) values ('||
                  v_m_jornada||','||''''||v_descripcio_jornada||''''||
                  ')';
    v_carrega_actual:=f_executa_instr(v_sentencia);              
    if (v_carrega_actual<>1) THEN  v_Errors_actual:= v_Errors_actual+1;END IF;                     
    --Nombre d'erros mxim superat
    IF(v_Errors_actual>v_Maxim_Errors) THEN RAISE e_Massa_Errors; END IF;
   FETCH c_TJornada into v_m_jornada,v_descripcio_jornada;
END LOOP;
CLOSE c_TJornada;   
dbms_output.put_line('F_CARREGAR_TIPUS_JORNADA:Taula Tipus_Jornada carregada.');
return 1;

EXCEPTION  
  WHEN e_Massa_Errors THEN
        insertar_log('carregar Tipus Jornada: Nombre d''errors al carregar la dimensio tipus Jornada massa gran. Abortar carrega magatzem');
        return -1;
                
  WHEN OTHERS THEN
         insertar_log('ABORTAR CARREGA TOTAL: carregar Tipus Jornada: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
         return -1; 
END F_CARREGAR_TIPUS_JORNADA;

/

--------------------------------------------------------
--  DDL for Function F_EXECUTA_INSTR
--------------------------------------------------------

  CREATE OR REPLACE FUNCTION "MAGATZEM"."F_EXECUTA_INSTR" 
/*Aquest funci executa la instruccio pasada per parametre*/
(
  PINSTR IN VARCHAR2  
) RETURN NUMBER AS 
BEGIN
EXECUTE IMMEDIATE PINSTR;
return 1;                     
EXCEPTION                          
  WHEN OTHERS THEN
         insertar_log('Executar Instr: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200)); 
         insertar_log('Executar Instr: '||PINSTR );                     
         return -1; 
END F_EXECUTA_INSTR;

/

--------------------------------------------------------
--  DDL for Procedure BUIDAR_MAGATZEM
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MAGATZEM"."BUIDAR_MAGATZEM" AS 
/*Aquest procediment esborra el contingut de totes les taules del magatzem.
  Per garantir que no hi haura problemes:
   1 - Desactiva totes les contraints.
   2 - Realitza esborrat
   3 - Activa constraints*/
BEGIN
--Desactivo les restriccions
desfer_constraints;

--Esborro totes les taules
execute immediate ('truncate table ciutat');
execute immediate ('truncate table dia');
execute immediate ('truncate table empresa');
execute immediate ('truncate table mes');
execute immediate ('truncate table mestre_rols');
execute immediate ('truncate table numany');
execute immediate ('truncate table oferta');
execute immediate ('truncate table pais');
execute immediate ('truncate table provincia');
execute immediate ('truncate table sector');
execute immediate ('truncate table setmana');
execute immediate ('truncate table sub_sector');
execute immediate ('truncate table tipus_contracte');
execute immediate ('truncate table tipus_jornada');

--Activo les constraints;
fer_constraints;
END BUIDAR_MAGATZEM;

/

--------------------------------------------------------
--  DDL for Procedure CARREGAR_MAGATZEM
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MAGATZEM"."CARREGAR_MAGATZEM" AS 
/*Aquest procediment recarrega el magatzem sencser
Per fer-ho primer esborra les dades del magatzem
Desprs carrega les dimensions i finalment la taula de fets*/
v_resultat NUMBER;
e_PararCarregaDimensions EXCEPTION;
e_PararCarregaFets EXCEPTION;
BEGIN
--Esborro totes les taules del magatzem
buidar_magatzem;

--Carreguem les dimensions
v_resultat:=f_carregar_dimensions;
IF (v_resultat<>1) THEN RAISE e_PararCarregaDimensions;END IF;

--Carrego la taula de fets
v_resultat:=f_calcular_fets_oferta;
IF (v_resultat<>1) THEN RAISE e_PararCarregaFets;END IF;
dbms_output.put_line('MAGATZEM CARREGAT');

EXCEPTION  
  WHEN e_PararCarregaDimensions THEN
        insertar_log('CARREGA ABORTADA. HI HA HAGUT PROBLEMES AL CARREGAR DIMENSIONS');
        dbms_output.put_line('CARREGA ABORTADA. HI HA HAGUT PROBLEMES AL CARREGAR DIMENSIONS');
  WHEN e_PararCarregaFets THEN
        insertar_log('CARREGA ABORTADA. HI HA HAGUT PROBLEMES AL CARREGAR FETS');
        dbms_output.put_line('CARREGA ABORTADA. HI HA HAGUT PROBLEMES AL CARREGAR FETS');
  WHEN OTHERS THEN
        insertar_log('ABORTAR CARREGA TOTAL: '|| 
                             SQLCODE ||  ' - '||
                             substr(SQLERRM, 1,200));  
        dbms_output.put_line('Anular carrega de magatzem. Problema no esperat');
        
END CARREGAR_MAGATZEM;

/

--------------------------------------------------------
--  DDL for Procedure DESFER_CONSTRAINTS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MAGATZEM"."DESFER_CONSTRAINTS" AS 
/*
Aquest procediment desfa totes les claus foranies
*/
BEGIN
execute immediate('alter table CIUTAT drop constraint FK_PROVINCIA_CIUTAT');

execute immediate('alter table DIA drop constraint "FK_Mes_Dia"');

execute immediate('alter table DIA drop constraint FK_REF_SETMANA_DIA');

execute immediate('alter table MES drop constraint FK_ANY_MES');

execute immediate('alter table OFERTA drop constraint FK_CIUTAT_OFERTA');

execute immediate('alter table OFERTA drop constraint "FK_Data_Alta_Oferta"');

execute immediate('alter table OFERTA drop constraint FK_OFERTA_REF_DATA__DIA');

execute immediate('alter table OFERTA drop constraint FK_EMPRESA_OFERTA');

execute immediate('alter table OFERTA drop constraint FK_REF_JORNADA_OFERTA');

execute immediate('alter table OFERTA drop constraint FK_REF_ROL_OFERTA');

execute immediate('alter table OFERTA drop constraint FK_SUB_SECTOR_OFERTA');

execute immediate('alter table OFERTA drop constraint FK_TIPUS_CONTRACTE_OFERTA');

execute immediate('alter table PROVINCIA drop constraint FK_PROVINCI_REF_PAIS__PAIS');

execute immediate('alter table SETMANA drop constraint FK_ANY_SETMANA');

execute immediate('alter table SUB_SECTOR drop constraint FK_SUB_SECT_REF_SECTO_SECTOR');

END DESFER_CONSTRAINTS;

/

--------------------------------------------------------
--  DDL for Procedure FER_CONSTRAINTS
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MAGATZEM"."FER_CONSTRAINTS" AS 
BEGIN
execute immediate('alter table CIUTAT add constraint FK_PROVINCIA_CIUTAT foreign key (ID_PAIS_PROVINCIA, ID_PROVINCIA_CIUTAT) references PROVINCIA (ID_PAIS_PROVINCIA, ID_PROVINCIA)');

execute immediate('alter table DIA add constraint "FK_Mes_Dia" foreign key (NUMANY, NUM_MES) references MES (NUMANY, NUM_MES)');

execute immediate('alter table DIA add constraint FK_REF_SETMANA_DIA foreign key (NUMANY, NUM_SETMANA) references SETMANA ("ANY", NUM_SETMANA)');

execute immediate('alter table MES add constraint FK_ANY_MES foreign key (NUMANY) references NUMANY (NUMANY)');

execute immediate('alter table OFERTA add constraint FK_CIUTAT_OFERTA foreign key (ID_PROVINCIA_CIUTAT, CIUTAT_TREBALL_OFERTA) references CIUTAT (ID_PROVINCIA_CIUTAT, ID_CIUTAT)');

execute immediate('alter table OFERTA add constraint "FK_Data_Alta_Oferta" foreign key (DATA_ALTA_OFERTA) references DIA (DIA)');

execute immediate('alter table OFERTA add constraint FK_OFERTA_REF_DATA__DIA foreign key (DATA_LIMIT) references DIA (DIA)');

execute immediate('alter table OFERTA add constraint FK_EMPRESA_OFERTA foreign key (ID_EMPRESA_OFERTA) references EMPRESA (ID_EMPRESA)');

execute immediate('alter table OFERTA add constraint FK_REF_JORNADA_OFERTA foreign key (TIPUS_JORNADA_OFERTA) references TIPUS_JORNADA (M_JORNADA)');

execute immediate('alter table OFERTA add constraint FK_REF_ROL_OFERTA foreign key (ROL_CANDIDAT_OFERTA) references MESTRE_ROLS (ID_ROL)');
      
execute immediate('alter table OFERTA add constraint FK_SUB_SECTOR_OFERTA foreign key (ID_SECTOR, SUB_SECTOR_EMPRESA) references SUB_SECTOR (ID_SECTOR, ID_SUB_SECTOR)');

execute immediate('alter table OFERTA add constraint FK_TIPUS_CONTRACTE_OFERTA foreign key (TIPUS_CONTRACTE_OFERTA) references TIPUS_CONTRACTE (M_CONTRACTE)');

execute immediate('alter table PROVINCIA add constraint FK_PROVINCI_REF_PAIS__PAIS foreign key (ID_PAIS_PROVINCIA) references PAIS (ID_PAIS)');

execute immediate('alter table SETMANA add constraint FK_ANY_SETMANA foreign key ("ANY") references NUMANY (NUMANY)');

execute immediate('alter table SUB_SECTOR add constraint FK_SUB_SECT_REF_SECTO_SECTOR foreign key (ID_SECTOR) references SECTOR (ID_SECTOR)');

END FER_CONSTRAINTS;

/

--------------------------------------------------------
--  DDL for Procedure INSERTAR_LOG
--------------------------------------------------------
set define off;

  CREATE OR REPLACE PROCEDURE "MAGATZEM"."INSERTAR_LOG" 
(
  P_DESCRIPCIO IN taula_log.descripcio_problema%TYPE  
) AS 
BEGIN
INSERT INTO taula_log (id_log, data_alta, descripcio_problema)values 
            (seq_log.nextval,to_date(sysdate,'DD/MM/YYYY HH24:MI:SS')
            ,p_descripcio);
END INSERTAR_LOG;

/

